<?php

/**
 * Classe per interagire con gCloud Monitoring
 */
require_once 'HTTP/Request2.php';
require_once 'GELF/GELFMessage.php';
require_once 'GELF/GELFMessagePublisher.php';

class gCloud_Monitoring {

    /**
     * @var string Attuale versione della libreria
     * @access public
     */
    const VERSION = "gCloud_Monitoring 1.0.1 [Pear]";

    /**
     * @var string nome dello stream
     * @access public
     */
    public $streamName = NULL;

    /**
     * @var string chiave di autenticazione
     * @access public
     */
    public $authentication = NULL;

    /**
     * @var string protocollo da utilizzare per l'invio del log (REST / GELF)
     * @access public
     */
    public $protocol = NULL;

    /**
     * @var string endpoint del servizio REST (hardcoded a "monitoring.gcloud.schema31.it)
     * @access public
     */
    public $restEndPoint = "monitoring.gcloud.schema31.it";

    /**
     * @var string endpoint del servizio GELF (hardcoded a "gelf.gcloud.schema31.it)
     * @access public
     */
    public $gelfEndPoint = "gelf.gcloud.schema31.it";

    /**
     * Variabili di appoggio per la response (http status code, header e body di risposta)
     * @access public
     */
    public $responseStatus = NULL;
    public $responseHeaders = NULL;
    public $responseBody = NULL;

    /**
     * Attributi del sender
     * @var string 
     * @access private
     */
    private $clientHostname = NULL;
    private $clientBrowser = NULL;
    private $clientIP = NULL;
    private $realClientIP = NULL;

    /**
     * Attributi del log
     * @var string 
     * @access private
     */
    private $shortMessage = NULL;
    private $fullMessage = NULL;
    private $logLevel = NULL;
    private $facility = NULL;
    private $file = NULL;
    private $line = NULL;
    private $additionals = array();

    /**
     * Variabile di appoggio per l'ultimo errore ricevuto
     *
     * @var string 
     * @access public
     */
    public $lastError = "";

    public function __construct($streamName = '', $authentication = '', $protocol = '') {

        $this->streamName = $streamName;
        $this->authentication = $authentication;
        $this->protocol = (strtoupper($protocol) == 'REST' ? 'REST' : 'GELF');

        $this->clientHostname = php_uname('n');
        $this->clientBrowser = (isset($_SERVER) && isset($_SERVER['HTTP_USER_AGENT']) ? $_SERVER['HTTP_USER_AGENT'] : '');
        $this->clientIP = (isset($_SERVER) && isset($_SERVER['REMOTE_ADDR']) ? $_SERVER['REMOTE_ADDR'] : '');
        $this->realClientIP = (isset($_SERVER) && isset($_SERVER['HTTP_X_FORWARDED_FOR']) && isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER['HTTP_X_FORWARDED_FOR'] . ' via ' . $_SERVER["REMOTE_ADDR"] : '');
    }

    /**
     * Inizializza le variabili di stato prima di effettuare una chiamata
     * @return void
     */
    private function init() {

        $this->responseBody = NULL;
        $this->responseHeaders = NULL;
        $this->responseStatus = NULL;
        $this->lastError = NULL;
    }

    /**
     * Effettua il salvataggio di un log
     * @param string $shortMessage messaggio breve (ricercabile)
     * @param string $fullMessage testo libero (non ricercabile)
     * @param int $logLevel (ricercabile) - 0 = EMERGENCY, 1 = ALERT, 2 = CRITICAL, 3 = ERROR, 4 = WARNING, 5 = NOTICE, 6 = INFO, 7 = DEBUG
     * @param string $facility
     * @param string $file nome del file che sta richiamando questo metodo
     * @param int $line numero di riga del file che sta richiamando questo metodo
     * @param array $logMessageAdditionals array chiave / valore di parametri aggiuntivi
     * @return boolean esito operazione
     */
    public function sendLog($shortMessage = NULL, $fullMessage = NULL, $logLevel = NULL, $facility = NULL, $file = NULL, $line = NULL, $logMessageAdditionals = array()) {

        $this->init();

        /**
         * Validazione parametri obbligatori
         */
        if (strlen($shortMessage) == 0) {
            $this->lastError = "parametro 'shortMessage' non valorizzato";
            return FALSE;
        } elseif (strlen($logLevel) == 0 || !in_array($logLevel, array(0, 1, 2, 3, 4, 5, 6, 7))) {
            $this->lastError = "parametro 'logLevel' non valorizzato o non valido";
            return FALSE;
        } elseif (strlen($facility) == 0) {
            $this->lastError = "parametro 'facility' non valorizzato";
            return FALSE;
        }

        /**
         * Se il parametro '$shortMessage'  eccede la dimensione massima consentita, copiamo il contenuto nel '$fullMessage' per evitare perdita di informazioni
         */
        if (strlen($shortMessage) > 255) {
            $fullMessage = $shortMessage . PHP_EOL . PHP_EOL . PHP_EOL . PHP_EOL . $fullMessage;
        }

        /**
         * Settiamo gli attributi del log
         */
        $this->shortMessage = $shortMessage;
        $this->fullMessage = $fullMessage;
        $this->logLevel = $logLevel;
        $this->facility = $facility;
        $this->file = $file;
        $this->line = $line;

        /**
         * Formattiamo gli eventuali parametri aggiuntivi (accettiamo solo coppie chiave / valore)
         */
        $this->additionals = array();
        if (is_array($logMessageAdditionals) && count($logMessageAdditionals) > 0) {
            foreach ($logMessageAdditionals as $key => $value) {
                $this->additionals[$key] = (string) $value;
            }
        }

        /**
         * Aggiungiamo gli attributi del sender
         */
        if (strlen($this->clientBrowser)) {
            $this->additionals['Client Browser'] = $this->clientBrowser;
        }

        if (strlen($this->clientIP)) {
            $this->additionals['Client IP'] = $this->clientIP;
        }

        if (strlen($this->realClientIP)) {
            $this->additionals['Real Client IP'] = $this->realClientIP;
        }

        /**
         * Aggiungiamo la versione della libreria
         */
        $this->additionals['Library Version'] = self::VERSION;

        /**
         * Effettua l'invio del log, in base al protocollo selezionato
         */
        switch ($this->protocol) {
            case 'REST':
                return $this->sendRestLog();
            case 'GELF':
                return $this->sendGelfLog();
            default:
                return FALSE;
        }
    }

    /**
     * Effettua il salvataggio di un log (chiamata REST)
     * @return boolean esito operazione
     */
    private function sendRestLog() {

        /**
         * prepariamo il body per la richiesta
         */
        $body = array(
            'host' => $this->clientHostname,
            'short_message' => $this->shortMessage,
            'full_message' => $this->fullMessage,
            'level' => $this->logLevel,
            'facility' => $this->facility,
            'file' => $this->file,
            'line' => $this->line
        );

        /**
         * Il nome degli eventuali parametri aggiuntivi deve iniziare con '_'
         */
        foreach ($this->additionals as $key => $value) {
            $body["_{$key}"] = (string) $value;
        }

        /**
         * Creiamo un'istanza dell'oggetto Request2 definendo tutti i parametri
         */
        $request = new HTTP_Request2("http://{$this->restEndPoint}/api/gelflog");
        $request->setMethod(HTTP_Request2::METHOD_POST)
                ->setHeader('AuthKey', $this->authentication)
                ->setHeader('streamName', $this->streamName)
                ->setHeader('Expect', "")
                ->setBody(json_encode($body));

        /**
         *  Catturiamo la risposta
         */
        $HTTPRequest2Response = $request->send();
        $this->responseBody = $HTTPRequest2Response->getBody();
        $this->responseHeaders = $HTTPRequest2Response->getHeader();
        $this->responseStatus = $HTTPRequest2Response->getStatus();

        /**
         * Validazione http status code
         */
        if ($this->responseStatus != 201) {
            $this->lastError = "Errore esecuzione richiesta: {$this->responseStatus}";
            return FALSE;
        }

        return TRUE;
    }

    /**
     * Effettua il salvataggio di un log  (chiamata GELF)
     * @return boolean esito operazione
     */
    private function sendGelfLog() {

        /**
         * Inizializziamo l'oggetto della classe 'GELFMessage'
         */
        $GELFMessage = new \gCloud\GELFLog\GELFMessage();
        $GELFMessage->setHost($this->streamName);
        $GELFMessage->setAuthentication($this->authentication);
        $GELFMessage->setFacility($this->facility);
        $GELFMessage->setFile($this->file);
        $GELFMessage->setLine($this->line);
        $GELFMessage->setShortMessage($this->shortMessage);
        $GELFMessage->setFullMessage($this->fullMessage);
        $GELFMessage->setLevel($this->logLevel);

        /**
         * Aggiungiamo gli eventuali 'additionals'
         */
        foreach ($this->additionals as $key => $value) {
            $GELFMessage->setAdditional($key, $value);
        }

        /**
         * Il GELF server, per retrocompatibilità, riceverà il nome dello stream 
         * come attributo 'host', per tenere traccia del vero hostname del 
         * mittente, lo aggiungiamo agli 'additional'
         */
        $GELFMessage->setAdditional('Hostname', $this->clientHostname);

        /**
         * Inizializziamo l'oggetto della classe 'GELFMessagePublisher'
         */
        $GELFMessagePublisher = new \gCloud\GELFLog\GELFMessagePublisher($this->gelfEndPoint, 12201);

        /**
         * Validazione esito
         */
        return $GELFMessagePublisher->publish($GELFMessage);
    }

}
